# Flow

General purpose graph package for Go

## Initial Idea

`todo`

> Explain the initial Idea came from machine learning Tensorflow etc...

> And by watching drone CI stages, I saw I could implement timing and triggers
> on nodes creating the possibility to add Go functions that handle
> docker containers to execute our steps and triggers would execute nodes that
> handle notifications (as plugin/slack, plugin/email)

* Machine learning
* Graphical CI pipeline

## flow package

Using flow package without UI

```go
package main

import (
	"flow"
	"flow/registry"
	"log"
	"strings"
)

func main() {

	r := registry.New()
	r.Add(strings.Split, strings.Join)

	f := flow.New()
	f.SetRegistry(r)

	op := f.Op("Join",
		f.Op("Split", "hello world", " "),
		",",
	)

	res, err := op.Process()
	if err != nil {
		log.Fatal(err)
	}
	log.Println("res:", res)

}
```

## How it works

A flow works by requesting the previous nodes the results of its operation,
so the starting node will be the node we want the result of, unattached nodes wont be executed

### Registering a function

All nodes are Go functions to create a node src simply add a function like:

```go
package main

import (
	"flow/flowserver"
	"flow/registry"
	"net/http"
)

func main() {

	r := registry.New()
	r.Add("hello", func() string {
		return "hello world"
	})

	http.ListenAndServe(":5000", flowserver.New(r, "storename"))

}
```

Resulting in

![fig1](img/s1.jpg)

### Go funcs

Since every node is a function we can register functions from any Go
package

```go
// Adding functions from a go package
r.Add(strings.Split, strings.Join)
```

![fig2](img/gofuncs.jpg)

### Describing funcs

We can describe the inserted functions either by using `r.Add` methods or
grouping functions with the helper `registry.Describer`

```go
// utility to apply functions to several entries
registry.Describer(
	r.Add(strings.Split).Inputs("str", "sep").Output("slice"),
	r.Add(strings.Join).Inputs("slice", "sep").Output("string"),
).Tags("strings").Extra("style", registry.M{"color": "#a77"})
```

![fig3](img/describing.jpg)

### Results

![fig4](img/runningsplit.jpg)
![fig5](img/runningjoin.jpg)

---

> WIP

## TODO

### UX/UI

* UX/UI: Create Undo behaviour
* UX/UI: Special visualisers/nodes to display information (images, datatables, graphs)
* UX/UI: Ability to group nodes into a single box exposing inputs and outputs
* UX/UI: Implement touch
* UX/UI: Drop link in node to link to the next compatible input

### Packages

* FlowPkg: Create training mechanism
* FlowPkg: Machine learning examples

### Backend

* Backend: Export/Deploy a model to a training cluster
* Backend: Expose the model possibly via an Rest API for easy function access

### Other

* Collaboration: Better concurrent editing/message passing

## Ideas

### Matching types

in UI we can create a state such as draggingType{in:'io.Writer'} which basically
each node can figure a match for each socket

in Server side we can do an analysis on each type to see if it can be
implemented by another type and send this as part of the registry with all
descriptions, this way we can highlight sockets that can implement the dragging
type
